Vue.component('lct-form', {
    props: {
        method: String,
        submit: Function,
        id: String,
        customValidator: Function
    },
    data: function () {
        return {
            form: {},
            sending: false,
            error: false,
            errorValidation: false,

        }
    },
    methods: {
        setForm: function (form) {
            this.form = form;
            this.$forceUpdate();
            // console.log('setForm', this.form);
        },
        submitForm: function (event) {
            console.log('submit form');
            event.preventDefault();
            if (this.validateAll()) {
                this.submit(event);
            } else {
                console.log('validation fail');
            }
        },
        validateAll: function (force) {
            var valid = true;

            for (var i in this.$children) {
                var child = this.$children[i];
                /*
                if (!child.validate(force)) {
                    console.log('validate 1');
                    valid = false;
                }*/
                switch (child.$options.name) {
                    case "lct-input":
                    case "lct-predictivo":
                    case "lct-select":
                    case "lct-textarea":
                        if (!child.validate(force)) {
                            valid = false;
                        }
                        break;
                }
            }
            this.errorValidation = valid;
            // console.log('validateAll', valid);

            if (this.customValidator && !this.customValidator()) {
                valid = false;
            }

            return valid;
        },
        reset: function () {
            this.form = {};
            this.sending = false;
            this.error = false;
            this.clearErrors();
        },
        clearErrors: function () {
            for (var i in this.$children) {
                var child = this.$children[i];
                switch (child.$options.name) {
                    case "lct-input":
                    case "lct-predictivo":
                    case "lct-select":
                    case "lct-textarea":
                        child.clear();
                        break;
                }
            }
        },
        checkRelated: function () {
            for (var i in this.$refs) {
                var ref = this.$refs[i];
                for (var r in ref.rules) {
                    var split = ref.rules[r].split(':');
                    switch (split[0]) {
                        case "<=":
                        case ">=":
                        case "==":
                            ref.validate(true);
                            break;
                    }
                }
            }


        },
        hasError: function () {
            for (var i in this.$children) {
                var child = this.$children[i];
                if (!child.valid) {
                    return true;
                }
            }
            return false
        }
    },
    watch: {
        form: {
            deep: true,
            handler() {
                // console.log('El form cambio!', this.form);
            }
        }
    },
    mounted: function () {

    },
    template: `
    <form :id="id" :method="method" @submit="submitForm">
        <slot></slot>
    </form>
`
});

Vue.component('lct-input', {
    props: {
        name: String,
        rules: Array,
        type: String,
        value: {
            required: true
        },
        readonly: {
            type: Boolean,
            default: false
        },
        autocomplete: String,
        placeholder: String,
        onChange: {
            type: Function,
            default: function () {
            }
        },
        maxlength: null,
        addStyle: null,
        validClass: {
            type: String,
            default: 'is-valid'
        },
        invalidClass: {
            type: String,
            default: 'is-invalid'
        },
    },
    data: function () {
        return {
            localvalue: null,
            valid: null,
            validated: false,
            errors: []
        }
    },
    methods: {
        change: function () {
            this.validate();
        },
        clear: function () {
            this.valid = null;
            this.errors = [];
        },
        validate: function (stopPropagation) {

            this.errors = [];
            for (var i in this.rules) {
                var rule = this.rules[i].split(':');
                var type = rule[0];

                var value = this.localvalue;

                if (value === undefined) {
                    console.warn('Debe definir un valor inicial para los campos del formulario.');
                }


                switch (type) {
                    case "required":
                        if (!this.validateRequired(value)) {
                            this.errors.push({msg: 'El campo es obligatorio.'});
                        }
                        break;
                    case "number":
                        if (value != '' && value != null && !this.validateNumber(value)) {
                            this.errors.push({msg: 'El campo solo puede contener números.'});
                        }
                        break;
                    case "decimal":
                        if (value != '' && value != null && !this.validateDecimal(value)) {
                            this.errors.push({msg: 'El campo solo puede contener números enteros o con decimales.'});
                        }
                        break;
                    case "positive":
                        if (value != '' && value != null && !this.validatePositive(value)) {
                            this.errors.push({msg: 'El campo solo puede contener números positivos.'});
                        }
                        break;
                    case "positive-decimal":
                        if (value != '' && value != null && !this.validatePositiveDecimal(value)) {
                            this.errors.push({msg: 'El campo solo puede contener números positivos.'});
                        }
                        break;
                    case "minNumber":
                        if (value != '' && value != null && !this.validateMinNumber(value, rule[1])) {
                            this.errors.push({msg: 'El valor mínimo debe ser ' + rule[1] + '.'});
                        }
                        break;
                    case "maxNumber":
                        if (value != '' && value != null && !this.validateMaxNumber(value, rule[1])) {
                            this.errors.push({msg: 'El valor máximo debe ser ' + rule[1] + '.'});
                        }
                        break;
                    case "price":
                        if (value != '' && value != null && !this.validatePrice(value)) {
                            this.errors.push({msg: 'Precio con 2 decimales máximo, separados por punto.'});
                        }
                        break;
                    case "email":
                        if (!this.validateEmail(value)) {
                            this.errors.push({msg: 'Ingrese un email válido.'});
                        }
                        break;
                    case "max":
                        if (!this.validateMax(value, rule[1])) {
                            this.errors.push({msg: 'Superó el máximo de ' + rule[1] + ' carácteres.'});
                        }
                        break;
                    case "min":
                        if (!this.validateMin(value, rule[1])) {
                            this.errors.push({msg: rule[1] + ' carácteres como mínimo.'});
                        }
                        break;
                    case "<=":
                        if (!this.validateLessOrEqual(value, rule[1])) {
                            this.errors.push({msg: 'El campo debe ser menor o igual a ' + this.$parent.$refs[rule[1]].value + ' .'});
                        }
                        break;
                    case ">=":
                        if (!this.validateGreaterOrEqual(value, rule[1])) {
                            this.errors.push({msg: 'El campo debe ser mayor o igual a ' + this.$parent.$parent.$refs[rule[1]].value + '.'});
                        }
                        break;
                    case "==":
                        if (!this.validateEqual(value, rule[1])) {
                            this.errors.push({msg: 'Debe coincidir con ' + rule[1] + ' .'});
                        }
                        break;
                    case "date":
                        if (!this.validateDate(value)) {
                            this.errors.push({msg: 'Debe ingresar una fecha válida (DD-MM-AAAA).'});
                        }
                        break;

                }
            }
            if (this.errors.length) {
                this.valid = false;
            } else {
                this.valid = true;
            }

            if (!stopPropagation && this.$parent.checkRelated) {
                this.$parent.checkRelated(this);
            }

            return this.valid;
        },
        validateEqual: function (value, ref) {
            return value == this.$parent.$parent.$refs[ref].value;
        },
        validateLessOrEqual: function (value, ref) {
            return parseFloat(value) <= parseFloat(this.$parent.$parent.$refs[ref].value);
        },
        validateGreaterOrEqual: function (value, ref) {
            return parseFloat(value) >= parseFloat(this.$parent.$parent.$refs[ref].value);
        },
        validateMin: function (value, min) {
            return !value || value.length >= parseInt(min);
        },
        validateMax: function (value, max) {
            return !value || value.length <= parseInt(max);
        },
        validateRequired: function (value) {
            return value !== null && value !== '' && String(value).length > 0;
        },
        validateNumber: function (value) {
            return /^\-?\d+?$/.test(value);
        },
        validateDecimal: function (value) {
            return /^\-?\d+((\.)\d+)?$/.test(value);
        },
        validatePositive: function (value) {
            return this.validateNumber(value) && value >= 0;
        },
        validatePositiveDecimal: function (value) {
            return this.validateDecimal(value) && value >= 0;
        },
        validateMinNumber: function (value, min) {
            return this.validateNumber(value) && parseFloat(value) >= parseFloat(min);
        },
        validateMaxNumber: function (value, max) {
            return this.validateNumber(value) && parseFloat(value) <= parseFloat(max);
        },
        validatePrice: function (value) {
            return /^\d+(\.\d{1,2})?$/.test(String(value));
        },
        validateEmail: function (value) {
            return /[\w-]+@([\w-]+\.)+[\w-]+/.test(value);
        },
        validateDate: function (value) {
            var d = value.split('-');
            if (d.length == 3) {
                if (d[2].length != 4) {
                    return false;
                }
                var date = moment([d[2], d[1] - 1, d[0]]);
                return date.isValid();
            } else {
                return false;
            }
        },
        focus: function () {
            this.$refs.input.focus();
        },
        blur: function () {
            this.onBlur();
        },
        getValidationClass: function () {
            if (this.valid === false) {
                return this.invalidClass;
            }
            if (this.valid) {
                return this.validClass;
            }
        },
        setError: function (msg) {
            this.valid = false;
            this.errors.push({msg: 'El número de Tarjeta no es Válido.'});
        }
    },
    watch: {
        localvalue(newValue, oldValue) {
            this.$emit('input', newValue);
            this.onChange();
            if (newValue != undefined) {
                this.validate();
            }
        },
        value(newValue, oldValue) {
            this.localvalue = newValue;
            if (newValue != undefined) {
                this.validate();
            }
        }
    },
    mounted: function () {
        this.localvalue = this.value;
    },
    template: `
    <div style="width:100%;">
        <input class="form-control" :type="type" :name="name" :readonly="readonly"
            :class="getValidationClass()" v-model="localvalue" :autocomplete="autocomplete" :placeholder="placeholder" ref="input" :maxlength="maxlength" :style="addStyle">
        <div class="invalid-feedback" v-if="valid === false">
            {{ errors[0].msg }}
        </div>
    </div>
    `
});

Vue.component('lct-select', {
    props: {
        name: String,
        required: Boolean,
        value: {
            required: true
        },
        readonly: {
            type: Boolean,
            default: false
        },
    },
    data: function () {
        return {
            localvalue: '',
            valid: null,
            initialState: true

        }
    },
    methods: {
        clear: function () {
            this.valid = null;
            this.localvalue = '';
            this.initialState = true;
        },
        validate: function () {
            if (!this.initialState) {
                if (this.required) {
                    this.valid = this.localvalue != '';
                } else {
                    this.valid = true;
                }
                return this.valid;
            } else {
                this.initialState = false;
            }
        },
    },
    updated: function () {
        if (this.initialState) {
            this.initialState = false;
        }
    },
    mounted: function () {
        // console.log('select value', this.value);
        this.localvalue = this.value;

    },
    watch: {
        localvalue(newValue, oldValue) {
            this.$emit('input', newValue);
            //if (newValue != undefined) {
            this.validate();
            //}
        },
        value(newValue, oldValue) {
            // console.log('watch', newValue);
            this.localvalue = newValue;
            //if (newValue != undefined) {
            this.validate();
            //}
        }
    },
    template: `
    <div>
        <select class="form-control" :name="name" v-model="localvalue" :readonly="readonly">
            <slot></slot>
        </select>
        <div class="invalid-feedback" v-if="valid === false">
            El campo es obligatorio.
        </div>
    </div>
`
});

Vue.component('lct-textarea', {
    props: {
        name: String,
        required: Boolean,
        value: {
            required: true
        },
        addStyle: {
            type: String,
            default: null
        }
    },
    data: function () {
        return {
            localvalue: '',
            valid: null,
            initialState: true

        }
    },
    methods: {
        clear: function () {
            this.valid = null;
            this.$emit('input', '');
        },
        validate: function (force) {
            // console.log('validate force',force);
            if (this.required) {
                if (force) {
                    this.valid = this.localvalue != '';
                } else {
                    this.valid = this.localvalue == '' ? null : true;
                }
            } else {
                this.valid = true;
            }
            return this.valid;
        },
    },
    watch: {
        localvalue(newValue, oldValue) {
            if (newValue != oldValue) {
                // console.log('localvalue', 'new ' + newValue, 'old ' + oldValue);
                this.$emit('input', newValue);
                this.validate();
            }
        },
        value(newValue, oldValue) {
            // console.log('localvalue', 'new ' + newValue, 'old ' + oldValue);
            if (newValue != oldValue) {
                this.localvalue = newValue;
                this.validate();
            }
        }
    },
    template: `
    <div>
        <textarea class="form-control" :class="{'is-invalid': valid === false, 'is-valid': valid }" :name="name" v-model="localvalue" :style="addStyle">
        </textarea>
        <div class="invalid-feedback" v-if="valid === false">
            El campo es obligatorio.
        </div>
    </div>
`
})